pub enum ErrorKind {
    InvalidEndpointError,
    JsonParseError,
    DatabaseError,
}

pub struct Error {
    kind: ErrorKind,
    desc: Option<String>,
}

impl Error {
    pub fn new(kind: ErrorKind) -> Error {
        Error { kind, desc: None }
    }
}

impl ToString for Error {
    fn to_string(&self) -> String {
        let mut error = match self.kind {
            ErrorKind::InvalidEndpointError => "invalid endpoint",
            ErrorKind::JsonParseError => "unable to parse JSON data",
            ErrorKind::DatabaseError => "unable to connect to database",
        }
        .to_string();
        if let Some(desc) = &self.desc {
            error += ": ";
            error += desc;
        }
        error
    }
}

impl From<serde_json::Error> for Error {
    fn from(error: serde_json::Error) -> Self {
        Error {
            kind: ErrorKind::JsonParseError,
            desc: Some(error.to_string()),
        }
    }
}

impl From<r2d2::Error> for Error {
    fn from(error: r2d2::Error) -> Self {
        Error {
            kind: ErrorKind::DatabaseError,
            desc: Some(error.to_string()),
        }
    }
}
